@@ -18,8 +18,10 @@ cross_validation
 
 Available subpackages
 ---------------------
-test
+tests
     regression test suite
+solvers
+    solver implementations and auxillary functions
 
 Utilities
 ---------
@@ -33,7 +35,7 @@ __author__
 
 """
 
-from .api import manual, maximize, minimize, optimize
+from .api import manual, maximize, minimize, optimize, available_solvers
 from .api import wrap_call_log, wrap_constraints, make_solver, suggest_solver
 from .cross_validation import cross_validated, generate_folds
 from .parallel import pmap
@@ -45,4 +47,4 @@ __revision__ = "0.2.1"
 __all__ = ['manual', 'maximize', 'minimize', 'optimize',
            'wrap_call_log', 'wrap_constraints', 'make_solver',
            'suggest_solver', 'cross_validated', 'generate_folds',
-           'pmap']
+           'pmap', 'available_solvers']
