@@ -35,9 +35,12 @@
 from __future__ import print_function
 import json
 import sys
+import socket
 import itertools
 from . import functions
 from . import parallel
+import math
+
 
 import multiprocessing
 import threading
@@ -45,6 +48,37 @@ import threading
 __DEBUG = False
 
 
+__channel_in = sys.stdin
+__channel_out = sys.stdout
+
+
+def _find_replacement(key, kwargs):
+    """Finds a replacement for key that doesn't collide with anything in kwargs."""
+    key += '_'
+    while key in kwargs:
+        key += '_'
+    return key
+
+
+def _find_replacements(illegal_keys, kwargs):
+    """Finds replacements for all illegal keys listed in kwargs."""
+    replacements = {}
+    for key in illegal_keys:
+        if key in kwargs:
+            replacements[key] = _find_replacement(key, kwargs)
+    return replacements
+
+
+def _replace_keys(kwargs, replacements):
+    """Replace illegal keys in kwargs with another value."""
+
+    for key, replacement in replacements.items():
+        if key in kwargs:
+            kwargs[replacement] = kwargs[key]
+            del kwargs[key]
+    return kwargs
+
+
 def json_encode(data):
     """Encodes given data in a JSON string."""
     return json.dumps(data)
@@ -63,35 +97,83 @@ def json_decode(data):
     return json.loads(data)
 
 
-def send(data, channel=sys.stdout):
+def send(data):
     """Writes data to channel and flushes."""
-    print(data, file=channel)
-    channel.flush()
-    return None
+    print(data, file=__channel_out)
+    __channel_out.flush()
 
 
-def receive(channel=sys.stdin):
+def receive():
     """Reads data from channel."""
-    line = channel.readline()[:-1]
+    line = __channel_in.readline()[:-1]
     if not line:
         raise EOFError("Unexpected end of communication.")
     return line
 
 
+def open_socket(port, host='localhost'):
+    """Opens a socket to host:port and reconfigures internal channels."""
+    global __channel_in
+    global __channel_out
+    try:
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.bind(('', 0))
+        sock.connect((host, port))
+        __channel_in = sock.makefile('r')
+        __channel_out = sock.makefile('w')
+    except (socket.error, OverflowError, ValueError) as e:
+        print('Error making socket: ' + str(e), file=sys.stderr)
+        sys.exit(1)
+
+
+def open_server_socket():
+    serv_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    try:
+        serv_sock.bind(('', 0))
+    except socket.err as msg:
+        print('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
+        sys.exit()
+    serv_sock.listen(0)
+    port = serv_sock.getsockname()[1]
+    return port, serv_sock
+
+
+def accept_server_connection(server_socket):
+    global __channel_in
+    global __channel_out
+    try:
+        sock, _ = server_socket.accept()
+        __channel_in = sock.makefile('r')
+        __channel_out = sock.makefile('w')
+    except (socket.error, OverflowError, ValueError) as e:
+        print('Error making socket: ' + str(e), file=sys.stderr)
+        sys.exit(1)
+
+
 class EvalManager(object):
 
-    def __init__(self):
+    def __init__(self, max_vectorized=100, replacements={}):
+        """Constructs an EvalManager object.
+
+        :param max_vectorized: the maximum size of a vector evaluation
+            larger vectorizations will be chunked
+        :type max_vectorized: int
+        :param replacements: a mapping of `original:replacement` keyword names
+        :type replacements: dict
+
+        """
         # are we doing a parallel function evaluation?
         self._vectorized = False
+        self._max_vectorized = max_vectorized
 
         # the queue used for parallel evaluations
         self._queue = None
 
         # lock to use to add evaluations to the queue
-        self._add_lock = multiprocessing.Lock()
+        self._queue_lock = multiprocessing.Lock()
 
         # lock to get results
-        self._get_lock = multiprocessing.Lock()
+        self._result_lock = multiprocessing.Lock()
 
         # used to check whether Future started running
         self._semaphore = None
@@ -99,6 +181,20 @@ class EvalManager(object):
         # used to signal Future's to get their result
         self._processed_semaphore = None
 
+        # keys that must be replaced
+        self._replacements = dict((v, k) for k, v in replacements.items())
+
+
+    @property
+    def replacements(self):
+        """Keys that must be replaced: keys are current values, values are
+        what must be sent through the pipe."""
+        return self._replacements
+
+    @property
+    def max_vectorized(self):
+        return self._max_vectorized
+
     @property
     def cv(self):
         return self._cv
@@ -112,6 +208,27 @@ class EvalManager(object):
         return self._processed_semaphore
 
     def pmap(self, f, *args):
+        """Performs vector evaluations through pipes.
+
+        :param f: the objective function (piped_function_eval)
+        :type f: callable
+        :param args: function arguments
+        :type args: iterables
+
+        The vector evaluation is sent in chunks of size self.max_vectorized.
+
+        """
+        argslist = zip(*args)
+        results = []
+
+        # partition the vector evaluation in chunks <= self.max_vectorized
+        for idx in range(int(math.ceil(1.0 * len(argslist) / self.max_vectorized))):
+            chunk = argslist[idx * self.max_vectorized:(idx + 1) * self.max_vectorized]
+            results.extend(self._vector_eval(f, *zip(*chunk)))
+
+        return results
+
+    def _vector_eval(self, f, *args):
         self._queue = []
         self._vectorized = True
         self._semaphore = multiprocessing.Semaphore(0)
@@ -150,6 +267,9 @@ class EvalManager(object):
         return results
 
     def pipe_eval(self, **kwargs):
+        # fix python keywords back to original
+        kwargs = _replace_keys(kwargs, self.replacements)
+
         json_data = json_encode(kwargs)
         send(json_data)
 
@@ -161,12 +281,13 @@ class EvalManager(object):
         return decoded['value']
 
     def add_to_queue(self, **kwargs):
+        kwargs = _replace_keys(kwargs, self.replacements)
         try:
-            self.add_lock.acquire()
+            self.queue_lock.acquire()
             self.queue.append(kwargs)
             idx = len(self.queue) - 1
         finally:
-            self.add_lock.release()
+            self.queue_lock.release()
         return idx
 
     @property
@@ -178,20 +299,20 @@ class EvalManager(object):
         return self._queue
 
     @property
-    def add_lock(self):
-        return self._add_lock
+    def queue_lock(self):
+        return self._queue_lock
 
     def get(self, number):
         try:
-            self._get_lock.acquire()
+            self._result_lock.acquire()
             value = self._results[number]
         finally:
-            self._get_lock.release()
+            self._result_lock.release()
         return value
 
     def flush_queue(self):
         self._results = []
-        if self.queue:
+        if self._queue:
             json_data = json_encode(self.queue)
             send(json_data)
 
@@ -201,7 +322,6 @@ class EvalManager(object):
                 sys.stderr('ERROR: ' + decoded['error'])
                 sys.exit(1)
             self._results = decoded['values']
-        return None
 
 
 def make_piped_function(mgr):
